# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import json
from threading import local
from time import time

import six
from debug_toolbar import settings as dt_settings
from debug_toolbar.utils import get_stack, get_template_info, tidy_stacktrace
from django.utils.encoding import force_text
from sqlalchemy import event
from sqlalchemy.exc import CompileError


state = local()


class SQLAlchemyTracker(object):
    def __init__(self, engine, alias, logger):
        self.engine = engine
        self.alias = alias
        self.logger = logger

        self.register()

    def register(self):
        event.listen(self.engine, 'before_execute', self.before_execute)
        event.listen(self.engine, 'after_execute', self.after_execute)

    def unregister(self):
        event.remove(self.engine, 'before_execute', self.before_execute)
        event.remove(self.engine, 'after_execute', self.after_execute)

    def before_execute(self, conn, clause, multiparams, params):
        clause.start_time = time()

    def after_execute(self, conn, clause, multiparams, params, result):
        start_time = clause.start_time
        stop_time = time()
        duration = (stop_time - start_time) * 1000
        config = dt_settings.get_config()

        raw_sql = ' '.join(six.text_type(
            clause.compile(dialect=self.engine.dialect,
                           compile_kwargs={})
        ).splitlines())
        try:
            sql = ' '.join(six.text_type(
                clause.compile(dialect=self.engine.dialect,
                               compile_kwargs={'literal_binds': True})
            ).splitlines())
        except (CompileError, TypeError, NotImplementedError):
            # not all queries support literal_binds
            sql = raw_sql

        _params = ''
        try:
            _params = json.dumps(list(map(self._decode, params)))
        except Exception:
            pass  # object not JSON serializable

        if config['ENABLE_STACKTRACES']:
            stacktrace = tidy_stacktrace(reversed(get_stack()))
        else:
            stacktrace = []

        template_info = get_template_info()

        params = {
            'vendor': conn.dialect.name,
            'alias': self.alias,
            'sql': sql,
            'duration': duration,
            'raw_sql': raw_sql,
            'params': _params,
            'stacktrace': stacktrace,
            'start_time': start_time,
            'stop_time': stop_time,
            'is_slow': duration > config['SQL_WARNING_THRESHOLD'],
            'is_select': sql.lower().strip().startswith('select'),
            'template_info': template_info,
        }

        self.logger.record(**params)

        return params

    def _decode(self, param):
        try:
            return force_text(param, strings_only=True)
        except UnicodeDecodeError:
            return '(encoded string)'


def wrap_engine(engine, alias, panel):
    if not hasattr(state, 'trackers'):
        state.trackers = {}

    if alias in state.trackers:
        return
    state.trackers[alias] = SQLAlchemyTracker(engine, alias, panel)


def unwrap_engine(engine, alias):
    tracker = getattr(state, 'trackers', {}).pop(alias, None)
    if tracker:
        tracker.unregister()
        del tracker



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import re


def swap_fields(sql):
    expr = (
        r'(JOIN</strong> [\w\d\.]+'
        r'(?: <strong>AS</strong> [\w\d\.]+)? <strong>ON</strong> )'
        r'((?:[\w\d\.]+ = [&#;\'"\w\d\.]+(?: <strong>AND</strong> )?)+)'
    )
    subs = (
        r'\1'
        r'<a class="djDebugUncollapsed djDebugToggle" href="#">'
        r'&#8226;&#8226;&#8226;'
        r'</a> '
        r'<a class="djDebugCollapsed djDebugToggle" href="#">\2</a> '
    )
    return re.sub(expr, subs, sql)


def add_newlines(sql):
    statements = [
        '<strong>FROM</strong>',
        '<strong>JOIN</strong>',
        '<strong>LEFT OUTER JOIN</strong>',
        '<strong>RIGHT OUTER JOIN</strong>',
        '<strong>WHERE</strong>',
        '<strong>GROUP BY</strong>',
        '<strong>ORDER BY</strong>',
        '<strong>LIMIT</strong>',
    ]
    for s in statements:
        sql = sql.replace(s, '<br/>{}'.format(s))
    return sql



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

from debug_toolbar import settings as dt_settings
from debug_toolbar.panels import sql as panels
from django.core.exceptions import ImproperlyConfigured
from django.utils.module_loading import import_string

from .tracking import unwrap_engine, wrap_engine
from .utils import add_newlines, swap_fields


class SQLPanel(panels.SQLPanel):
    @property
    def engines(self):
        try:
            path = dt_settings.get_config()['ALCHEMY_DB_ALIASES']
        except KeyError:
            raise ImproperlyConfigured(
                'Must specify ALCHEMY_DB_ALIASES in DEBUG_TOOLBAR_CONFIG'
            )
        else:
            return import_string(path)()

    def enable_instrumentation(self):
        for alias, engine in self.engines.items():
            wrap_engine(engine, alias, self)

    def disable_instrumentation(self):
        for alias, engine in self.engines.items():
            unwrap_engine(engine, alias)

    def record_stats(self, stats):
        for q in stats.get('queries', []):
            if q.get('sql'):
                q['sql'] = add_newlines(swap_fields(q['sql']))
        super(SQLPanel, self).record_stats(stats)

    # TODO reimplement utility views. for now let them blow up
    # @classmethod
    # def get_urls(cls):
    #     return []



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals
import os

from setuptools import find_packages, setup

from debug_toolbar_alchemy import __author__, __version__


def read(fname):
    p = os.path.join(os.path.dirname(__file__), fname)
    with open(p, 'rb') as fid:
        return fid.read().decode('utf-8')


authors = read('AUTHORS.rst')
history = read('HISTORY.rst').replace('.. :changelog:', '')
licence = read('LICENSE.rst')
readme = read('README.rst')

requirements = read('requirements.txt').splitlines() + [
    'setuptools',
]

test_requirements = (
    read('requirements.txt').splitlines() +
    read('requirements-dev.txt').splitlines()[1:]
)

setup(
    name='django-debug-toolbar-alchemy',
    version=__version__,
    author=__author__,
    description='Django Debug Toolbar panel for SQLAlchemy.',
    long_description='\n\n'.join([readme, history, authors, licence]),
    url='https://github.com/miki725/django-debug-toolbar-alchemy',
    license='MIT',
    packages=find_packages(exclude=[]),
    install_requires=requirements,
    test_suite='tests',
    tests_require=test_requirements,
    keywords=' '.join([
        'django',
        'django-debug-toolbar',
        'sqlalchemy',
    ]),
    classifiers=[
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Development Status :: 2 - Pre-Alpha',
    ],
)
